/*
 * This file is part of Distributed John,
 * Copyright (C) 2001-2003 by Luis Parravicini
 * http://ktulu.com.ar
 */
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>
#include <stdlib.h>

#include "netutils.h"
#include "intl.h"

const char *err_list[] = {
	gettext_noop("No error"),
	gettext_noop("An error occurred sending data"),
	gettext_noop("An error occurred receiving data"),
	gettext_noop("Not all the data was sent"),
	gettext_noop("zero bytes read!"),
	gettext_noop("Timeout while waiting for server's answer"),
	gettext_noop("packet inconsistency")
};
int erridx;

void serialize_packet(struct generic_packet* pkt)
{
	pkt->size = htonl(pkt->size);
	switch (pkt->action) {
		case GET_WORK:
			break;
		case FIRST_WORD: {
			struct firstword* pkt2 = (struct firstword*)pkt;
			pkt2->unit_id = htonl(pkt2->unit_id);
			break;
			}
		case FINISH_WORK: {
			struct finishwork* pkt2 = (struct finishwork*)pkt;
			pkt2->unit_id = htonl(pkt2->unit_id);
			pkt2->cracked = htonl(pkt2->cracked);
			pkt2->length = htonl(pkt2->length);
			break;
			}
		case GET_CHARSET:
			break;
		case CHARSET_DATA:
			break;
		case NOW_SLEEP: {
			struct sleep* pkt2 = (struct sleep*)pkt;
			pkt2->secs = htonl(pkt2->secs);
			break;
			}
		case GET_CODE:
			break;
		case CODE_DATA: {
			struct codedata* pkt2 = (struct codedata*)pkt;
			pkt2->length = htonl(pkt2->length);
			break;
			}
	}
}

struct generic_packet* deserialize_packet(uint8_t* buf, unsigned int buf_len)
{
	struct generic_packet* pkt = (struct generic_packet*)buf;
	char* err_inconsist = "deserialize_packet: packet inconsistency "\
				"(got %u, header says %u)\n";
 

	pkt->size = ntohl(pkt->size);
	if (buf_len != pkt->size) {
		fprintf(stderr, err_inconsist, buf_len, pkt->size);
		return NULL;
	}
	
	switch (pkt->action) {
		case GET_WORK:
			break;
		case FIRST_WORD: {
			struct firstword* pkt2 = (struct firstword*)pkt;
			pkt2->unit_id = ntohl(pkt2->unit_id);
			if (buf_len != FIRSTWORD_SIZE(pkt)) {
				fprintf(stderr, err_inconsist, buf_len,
					pkt2->size);
				return NULL;
			}
			break;
			}
		case FINISH_WORK: {
			struct finishwork* pkt2 = (struct finishwork*)pkt;
			pkt2->unit_id = ntohl(pkt2->unit_id);
			pkt2->cracked = ntohl(pkt2->cracked);
			pkt2->length = ntohl(pkt2->length);
			if (pkt2->size != FINISHWORK_SIZE(pkt2)) {
				fprintf(stderr, err_inconsist, buf_len,
					pkt2->size);
				return NULL;
			}
			break;
			}
		case GET_CHARSET:
			break;
		case CHARSET_DATA:
			break;
		case NOW_SLEEP: {
			struct sleep* pkt2 = (struct sleep*)pkt;
			pkt2->secs = ntohl(pkt2->secs);
			break;
			}
		case GET_CODE:
			break;
		case CODE_DATA: {
			struct codedata* pkt2 = (struct codedata*)pkt;
			pkt2->length = ntohl(pkt2->length);
			break;
			}
		default:
			fprintf(stderr, "deserialize_packet: unknown action "\
					"code (%u)", pkt->action);
			return NULL;
	}
	return pkt;
}

int mysendto(int s, struct generic_packet* pkt, int flags,
	       const struct sockaddr_in *to)
{
	int ret;
	uint32_t size;

	pkt->size = packetsize(pkt->action, pkt);
	size = pkt->size;
	serialize_packet(pkt);
	ret = sendto(s, (void*)pkt, size, flags, (struct sockaddr*)to,
			sizeof(struct sockaddr_in));
	deserialize_packet((uint8_t*)pkt, size);
	return ret;
}

int mkserver(struct in_addr *listen, in_port_t port)
{
	int s;
	struct sockaddr_in sa;

	if ((s = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) == -1) {
		fprintf(stderr, _("Can't create socket\n"));
		return -1;
	}
	memset(&sa, 0, sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_addr = *listen;
	sa.sin_port = htons(port);
	if (bind(s, (struct sockaddr *)&sa, sizeof(sa)) == -1) {
		fprintf(stderr, _("Can't bind to %s\n"),inet_ntoa(sa.sin_addr));
		return -1;
	}

	return s;
}

int sendmessage(int s, struct generic_packet* pkt,
	const struct sockaddr_in *to,
	void* buf, int buf_len, int wait_for_answer)
{
	fd_set rfds;
	struct timeval tv;
	int leido, escrito;

	if ((escrito=mysendto(s, pkt, 0, to)) == -1)
		return erridx = ERR_SENDING;

	if (escrito != pkt->size)
		return erridx = ERR_LESS;

	if (wait_for_answer) {
		FD_ZERO(&rfds);
		FD_SET(s, &rfds);
		tv.tv_sec = 15;
		tv.tv_usec = 0;
		if (select(s+1, &rfds, NULL, NULL, &tv) == 1) {
			if ((leido=recvfrom(s, buf, buf_len, 0, NULL, NULL)) ==
			-1)
				return erridx = ERR_RECVING;

			if (leido < 1)
				return erridx = ERR_LESS2;
		} else
			return erridx = ERR_TIMEOUT;
		if (deserialize_packet(buf, leido) == NULL)
			return erridx = ERR_CONSISTENCY;
	} else
		leido = 0;

	erridx = 0;
	return leido;
}

int send_to_client(int socket, struct generic_packet* pkt,
		const struct sockaddr_in *sa)
{
	int written;

	if ((written=mysendto(socket, pkt, 0, sa)) == -1) {
		perror(_("send_to_client"));
		return -1;
	}
	if (written != pkt->size) {
		fprintf(stderr, _("send_to_client: couldn't send all the "\
			"data to the client.\n"));
		return -1;
	}

	return 0;
}

/**
 * Reads a message from the socket.
 * Returns the error code ( > 0 ) or 0 on success.
 */
int read_message(int sock, void *buf, int buf_len, struct sockaddr_in* sa)
{
	int leido;
	struct generic_packet* pkt;
	int tam = sizeof(struct sockaddr_in);

	if ((leido=recvfrom(sock, buf, buf_len, 0, (struct sockaddr*)sa,
	&tam)) == -1)
		return 1;

	if (leido < GENERIC_PACKET_SIZE)
		return 2;

	if (deserialize_packet(buf, leido) == NULL) {
		return 4;
	}

	pkt = (struct generic_packet *)buf;
	if (pkt->size != leido) {
		return 3;
	}

	return 0;
}

